/*	$NetBSD: imx6_i2c.c,v 1.3 2019/06/20 08:16:19 hkenken Exp $	*/

/*
 * Copyright (c) 2014 Ryo Shimizu <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * i.MX6 I2C Controller
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD: imx6_i2c.c,v 1.3 2019/06/20 08:16:19 hkenken Exp $");

#include "opt_imx.h"

#include <sys/bus.h>
#include <sys/device.h>
#include <sys/param.h>

#include <arm/imx/imx6var.h>
#include <arm/imx/imx6_reg.h>
#include <arm/imx/imx6_ccmvar.h>
#include <arm/imx/imxi2cvar.h>

/* ARGSUSED */
int
imxi2c_match(device_t parent __unused, struct cfdata *match __unused, void *aux)
{
	struct axi_attach_args *aa = aux;

	switch (aa->aa_addr) {
	case IMX6_AIPS2_BASE + AIPS2_I2C1_BASE:
	case IMX6_AIPS2_BASE + AIPS2_I2C2_BASE:
	case IMX6_AIPS2_BASE + AIPS2_I2C3_BASE:
		return 1;
	}

	return 0;
}

/* ARGSUSED */
void
imxi2c_attach(device_t parent __unused, device_t self, void *aux)
{
	struct axi_attach_args *aa = aux;
	struct imxi2c_softc *sc = device_private(self);

	if (aa->aa_size <= 0)
		aa->aa_size = I2C_SIZE;

	switch (device_unit(self)) {
	case 0:
		sc->sc_clk = imx6_get_clock("i2c1");
		break;
	case 1:
		sc->sc_clk = imx6_get_clock("i2c2");
		break;
	case 2:
		sc->sc_clk = imx6_get_clock("i2c3");
		break;
	}
	if (sc->sc_clk == NULL) {
		aprint_error(": couldn't get clock sata\n");
		return;
	}

	int error = clk_enable(sc->sc_clk);
	if (error) {
		aprint_error_dev(sc->sc_dev, "couldn't enable: %d\n", error);
		return;
	}

	imxi2c_set_freq(self, clk_get_rate(sc->sc_clk), 400000);
	imxi2c_attach_common(parent, self,
	    aa->aa_iot, aa->aa_addr, aa->aa_size, aa->aa_irq, 0);
}

